# vim: tabstop=4 shiftwidth=4 softtabstop=4
#
# Copyright 2013 Hewlett Packard Development Company, L.P.
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

"""
HPLeftHand HTTP Client
:Author: Walter A. Boring IV
:Description: This is the HTTP Client that is used to make the actual calls.
 It includes the authentication that knows the cookie name for LH.

"""

import logging
import httplib2
import time
import pprint

try:
    import json
except ImportError:
    import simplejson as json

from hplefthandclient import exceptions


class HTTPJSONRESTClient(httplib2.Http):
    """
    An HTTP REST Client that sends and recieves JSON data as the body of the
    HTTP request.

    :param api_url: The url to the LH OS REST service
                    ie. https://<hostname or IP>:<port>/lhos
    :type api_url: str
    :param insecure: Use https? requires a local certificate
    :type insecure: bool

    """

    USER_AGENT = 'python-hplefthandclient'
    SESSION_COOKIE_NAME = 'Authorization'
    #API_VERSION = 'X-API-Version'
    #CHRP_VERSION = 'X_HP-CHRP-Client-Version'

    def __init__(self, api_url, insecure=False, http_log_debug=False):
        super(HTTPJSONRESTClient,
              self).__init__(disable_ssl_certificate_validation=True)
        self.session_key = None

        #should be http://<Server:Port>/lhos
        self.set_url(api_url)
        self.set_debug_flag(http_log_debug)

        self.times = []  # [("item", starttime, endtime), ...]

        # httplib2 overrides
        self.force_exception_to_status_code = True
        #self.disable_ssl_certificate_validation = insecure

        self._logger = logging.getLogger(__name__)

    def set_url(self, api_url):
        #should be http://<Server:Port>/lhos
        self.api_url = api_url.rstrip('/')
        self.api_url = self.api_url

    def set_debug_flag(self, flag):
        """
        This turns on/off http request/response debugging output to console

        :param flag: Set to True to enable debugging output
        :type flag: bool

        """
        self.http_log_debug = flag
        if self.http_log_debug:
            ch = logging.StreamHandler()
            self._logger.setLevel(logging.DEBUG)
            self._logger.addHandler(ch)

    def authenticate(self, user, password, optional=None):
        """
        This tries to create an authenticated session with the LH OS server

        :param user: The username
        :type user: str
        :param password: The password
        :type password: str

        """
        #this prevens re-auth attempt if auth fails
        self.auth_try = 1
        self.session_key = None

        info = {'user': user, 'password': password}
        self._auth_optional = None

        if optional:
            self._auth_optional = optional
            info.update(optional)

        resp, body = self.post('/credentials', body=info)
        if body and 'authToken' in body:
            self.session_key = body['authToken']
        self.auth_try = 0
        self.user = user
        self.password = password

    def _reauth(self):
        self.authenticate(self.user, self.password, self._auth_optional)

    def unauthenticate(self):
        """
        This clears the authenticated session with the LH server.  It logs out.

        """
        #delete the session on the LH
        self.delete('/credentials/%s' % self.session_key)
        self.session_key = None

    def get_timings(self):
        """
        Ths gives an array of the request timings since last reset_timings call
        """
        return self.times

    def reset_timings(self):
        """
        This resets the request/response timings array
        """
        self.times = []

    def _http_log_req(self, args, kwargs):
        if not self.http_log_debug:
            return

        string_parts = ['curl -i']
        for element in args:
            if element in ('GET', 'POST'):
                string_parts.append(' -X %s' % element)
            else:
                string_parts.append(' %s' % element)

        for element in kwargs['headers']:
            header = ' -H "%s: %s"' % (element, kwargs['headers'][element])
            string_parts.append(header)

        self._logger.debug("\nREQ: %s\n" % "".join(string_parts))
        if 'body' in kwargs:
            self._logger.debug("REQ BODY: %s\n" % (kwargs['body']))

    def _http_log_resp(self, resp, body):
        if not self.http_log_debug:
            return
        self._logger.debug("RESP:%s\n", pprint.pformat(resp))
        self._logger.debug("RESP BODY:%s\n", body)

    def request(self, *args, **kwargs):
        """
        This makes an HTTP Request to the LH server.  You should use get, post,
        delete instead.
        """
        if self.session_key and self.auth_try != 1:
            kwargs.setdefault('headers',
                              {})[self.SESSION_COOKIE_NAME] = self.session_key

        kwargs.setdefault('headers', kwargs.get('headers', {}))
        kwargs['headers']['User-Agent'] = self.USER_AGENT
        kwargs['headers']['Accept'] = 'application/json'
        if 'body' in kwargs:
            kwargs['headers']['Content-Type'] = 'application/json'
            kwargs['body'] = json.dumps(kwargs['body'])

        self._http_log_req(args, kwargs)
        resp, body = super(HTTPJSONRESTClient, self).request(*args, **kwargs)
        self._http_log_resp(resp, body)

        # Try and conver the body response to an object
        # This assumes the body of the reply is JSON
        if body:
            try:
                body = json.loads(body)
            except ValueError:
                #pprint.pprint("failed to decode json\n")
                pass
        else:
            body = None

        if resp.status >= 400:
            raise exceptions.from_response(resp, body)

        return resp, body

    def _time_request(self, url, method, **kwargs):
        start_time = time.time()
        resp, body = self.request(url, method, **kwargs)
        self.times.append(("%s %s" % (method, url),
                           start_time, time.time()))
        return resp, body

    def _do_reauth(self, url, method, ex, **kwargs):
        print "_do_reauth called"
        try:
            if self.auth_try != 1:
                self._reauth()
                resp, body = self._time_request(self.api_url + url,
                                                method, **kwargs)
                return resp, body
            else:
                raise ex
        except exceptions.HTTPUnauthorized:
            raise ex

    def _cs_request(self, url, method, **kwargs):
        # Perform the request once. If we get a 401 back then it
        # might be because the auth token expired, so try to
        # re-authenticate and try again. If it still fails, bail.
        try:
            resp, body = self._time_request(self.api_url + url, method,
                                            **kwargs)
            return resp, body
        except exceptions.HTTPUnauthorized, ex:
            print "_CS_REQUEST HTTPUnauthorized"
            resp, body = self._do_reauth(url, method, ex, **kwargs)
            return resp, body
        except exceptions.HTTPForbidden, ex:
            print "_CS_REQUEST HTTPForbidden"
            resp, body = self._do_reauth(url, method, ex, **kwargs)
            return resp, body

    def get(self, url, **kwargs):
        """
        Make an HTTP GET request to the server.

        .. code-block:: python

            #example call
            try {
                headers, body = http.get('/volumes')
            } except exceptions.HTTPUnauthorized as ex:
                print "Not logged in"
            }

        :param url: The relative url from the LH api_url
        :type url: str

        :returns: headers - dict of HTTP Response headers
        :returns: body - the body of the response.  If the body was JSON,
        it will be an object
        """
        return self._cs_request(url, 'GET', **kwargs)

    def post(self, url, **kwargs):
        """
        Make an HTTP POST request to the server.

        .. code-block:: python

            #example call
            try {
                info = {'name': 'new volume name', 'sizeMiB': 300}
                headers, body = http.post('/volumes', body=info)
            } except exceptions.HTTPUnauthorized as ex:
                print "Not logged in"
            }

        :param url: The relative url from the LH api_url
        :type url: str

        :returns: headers - dict of HTTP Response headers
        :returns: body - the body of the response.  If the body was JSON,
        it will be an object
        """
        return self._cs_request(url, 'POST', **kwargs)

    def put(self, url, **kwargs):
        """
        Make an HTTP PUT request to the server.

        .. code-block:: python

            #example call
            try {
                info = {'name': 'something'}
                headers, body = http.put('/volumes', body=info)
            } except exceptions.HTTPUnauthorized as ex:
                print "Not logged in"
            }

        :param url: The relative url from the LH api_url
        :type url: str

        :returns: headers - dict of HTTP Response headers
        :returns: body - the body of the response.  If the body was JSON,
        it will be an object
        """
        return self._cs_request(url, 'PUT', **kwargs)

    def delete(self, url, **kwargs):
        """
        Make an HTTP DELETE request to the server.

        .. code-block:: python

            #example call
            try {
                headers, body = http.delete('/volumes/%s' % name)
            } except exceptions.HTTPUnauthorized as ex:
                print "Not logged in"
            }

        :param url: The relative url from the LH api_url
        :type url: str

        :returns: headers - dict of HTTP Response headers
        :returns: body - the body of the response.  If the body was JSON,
        it will be an object
        """
        return self._cs_request(url, 'DELETE', **kwargs)



# vim: tabstop=4 shiftwidth=4 softtabstop=4
# Copyright 2009-2012 10gen, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""Test base class of LeftHand Client"""

import sys
import os
sys.path.insert(0, os.path.realpath(os.path.abspath('../')))

from hplefthandclient import client
import unittest
import subprocess
import time
import inspect
from testconfig import config
from urlparse import urlparse

# pip install nose-testconfig

# e.g.
# nosetests test_HPLeftHandClient_volume.py -v --tc-file config.ini


class HPLeftHandClientBaseTestCase(unittest.TestCase):

    user = config['TEST']['user']
    password = config['TEST']['pass']
    cluster = config['TEST']['cluster']
    flask_url = config['TEST']['flask_url']
    url_lhos = config['TEST']['lhos_url']
    debug = config['TEST']['debug'].lower() == 'true'
    unitTest = config['TEST']['unit'].lower() == 'true'
    startFlask = config['TEST']['start_flask_server'].lower() == 'true'

    def setUp(self):

        cwd = os.path.dirname(os.path.abspath(
                              inspect.getfile(inspect.currentframe())))

        if self.unitTest:
            self.printHeader('Using flask ' + self.flask_url)
            self.cl = client.HPLeftHandClient(self.flask_url)
            parsed_url = urlparse(self.flask_url)
            userArg = '-user=%s' % self.user
            passwordArg = '-password=%s' % self.password
            portArg = '-port=%s' % parsed_url.port

            script = 'test_HPLeftHandMockServer_flask.py'
            path = "%s/%s" % (cwd, script)
            try:
                if self.startFlask:
                    self.mockServer = subprocess.Popen([sys.executable,
                                                        path,
                                                        userArg,
                                                        passwordArg,
                                                        portArg],
                                                       stdout=subprocess.PIPE,
                                                       stderr=subprocess.PIPE,
                                                       stdin=subprocess.PIPE)
                else:
                    pass
            except Exception:
                pass
            time.sleep(1)
        else:
            self.printHeader('Using LeftHand ' + self.url_lhos)
            self.cl = client.HPLeftHandClient(self.url_lhos)

        if self.debug:
            self.cl.debug_rest(True)

        self.cl.login(self.user, self.password)

    def tearDown(self):
        self.cl.logout()
        if self.unitTest and self.startFlask:
            #TODO: it seems to kill all the process except the last one...
            #don't know why
            self.mockServer.kill()

    def printHeader(self, name):
        print "\n##Start testing '%s'" % name

    def printFooter(self, name):
        print "##Compeleted testing '%s\n" % name

    def findInDict(self, dic, key, value):
        for i in dic:
            if key in i and i[key] == value:
                return True



# vim: tabstop=4 shiftwidth=4 softtabstop=4
#
# Copyright 2012 Hewlett Packard Development Company, L.P.
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.
""" 
Exceptions for the client

.. module: Exceptions

:Author: Walter A. Boring IV
:Description: This contains the HTTP exceptions that can come back from the REST calls
"""


class UnsupportedVersion(Exception):
    """
    Indicates that the user is trying to use an unsupported version of the API
    """
    pass

class CommandError(Exception):
    pass

class AuthorizationFailure(Exception):
    pass

class NoUniqueMatch(Exception):
    pass

class ClientException(Exception):
    """
    The base exception class for all exceptions this library raises.

    :param error: The error array 
    :type error: array

    """
    _error_code = None
    _error_desc = None

    _debug1 = None
    _debug2 = None

    def __init__(self, error=None):
        if error:
            if 'messageID' in error:
                self._error_code = error['messageID']
            if 'message' in error:
                self._error_desc = error['message']

            if 'debug1' in error:
                self._debug1 = error['debug1']
            if 'debug2' in error:
                self._debug2 = error['debug2']

    def get_code(self):
        return self._error_code

    def get_description(self):
        return self._error_desc


    def __str__(self):
        formatted_string = "%s (HTTP %s)" % (self.message, self.http_status)
        if self._error_code:
            formatted_string += " %s" % self._error_code
        if self._error_desc:
            formatted_string += " - %s" % self._error_desc

        if self._debug1:
            formatted_string += " (1: '%s')" % self._debug1

        if self._debug2:
            formatted_string += " (2: '%s')" % self._debug2
          
        return formatted_string


##
## 400 Errors
##


class HTTPBadRequest(ClientException):
    """
    HTTP 400 - Bad request: you sent some malformed data.
    """
    http_status = 400
    message = "Bad request"


class HTTPUnauthorized(ClientException):
    """
    HTTP 401 - Unauthorized: bad credentials.
    """
    http_status = 401
    message = "Unauthorized"


class HTTPForbidden(ClientException):
    """
    HTTP 403 - Forbidden: your credentials don't give you access to this
    resource.
    """
    http_status = 403
    message = "Forbidden"


class HTTPNotFound(ClientException):
    """
    HTTP 404 - Not found
    """
    http_status = 404
    message = "Not found"

class HTTPMethodNotAllowed(ClientException):
    """
    HTTP 405 - Method not Allowed 
    """
    http_status = 405
    message = "Method Not Allowed"

class HTTPNotAcceptable(ClientException):
    """
    HTTP 406 - Method not Acceptable
    """
    http_status = 406
    message = "Method Not Acceptable"

class HTTPProxyAuthRequired(ClientException):
    """
    HTTP 407 - The client must first authenticate itself with the proxy.
    """
    http_status = 407
    message = "Proxy Authentication Required"

class HTTPRequestTimeout(ClientException):
    """
    HTTP 408 - The server timed out waiting for the request.
    """
    http_status = 408
    message = "Request Timeout"


class HTTPConflict(ClientException):
    """
    HTTP 409 - Conflict: A Conflict happened on the server
    """
    http_status = 409
    message = "Conflict"

class HTTPGone(ClientException):
    """
    HTTP 410 - Indicates that the resource requested is no longer available and will not be available again.
    """
    http_status = 410
    message = "Gone"

class HTTPLengthRequired(ClientException):
    """
    HTTP 411 - The request did not specify the length of its content, which is required by the requested resource.
    """
    http_status = 411
    message = "Length Required"

class HTTPPreconditionFailed(ClientException):
    """
    HTTP 412 - The server does not meet one of the preconditions that the requester put on the request.
    """
    http_status = 412
    message = "Over limit"

class HTTPRequestEntityTooLarge(ClientException):
    """
    HTTP 413 - The request is larger than the server is willing or able to process
    """
    http_status = 413
    message = "Request Entity Too Large"

class HTTPRequestURITooLong(ClientException):
    """
    HTTP 414 - The URI provided was too long for the server to process.
    """
    http_status = 414
    message = "Request URI Too Large"

class HTTPUnsupportedMediaType(ClientException):
    """
    HTTP 415 - The request entity has a media type which the server or resource does not support.
    """
    http_status = 415
    message = "Unsupported Media Type"

class HTTPRequestedRangeNotSatisfiable(ClientException):
    """
    HTTP 416 - The client has asked for a portion of the file, but the server cannot supply that portion.
    """
    http_status = 416
    message = "Requested Range Not Satisfiable"

class HTTPExpectationFailed(ClientException):
    """
    HTTP 417 - The server cannot meet the requirements of the Expect request-header field.
    """
    http_status = 417
    message = "Expectation Failed"

class HTTPTeaPot(ClientException):
    """
    HTTP 418 - I'm a Tea Pot
    """
    http_status = 418
    message = "I'm A Teapot. (RFC 2324)"


##
## 500 Errors
##

class HTTPServerError(ClientException):
    """
    HTTP 500 - 
    """
    http_status = 500
    message = "Error"

# NotImplemented is a python keyword.
class HTTPNotImplemented(ClientException):
    """
    HTTP 501 - Not Implemented: the server does not support this operation.
    """
    http_status = 501
    message = "Not Implemented"

class HTTPBadGateway(ClientException):
    """
    HTTP 502 - The server was acting as a gateway or proxy and received an invalid response from the upstream server. 
    """
    http_status = 502
    message = "Bad Gateway"

class HTTPServiceUnavailable(ClientException):
    """
    HTTP 503 - The server is currently unavailable
    """
    http_status = 503
    message = "Service Unavailable"

class HTTPGatewayTimeout(ClientException):
    """
    HTTP 504 - The server was acting as a gateway or proxy and did not receive a timely response from the upstream server.
    """
    http_status = 504
    message = "Gateway Timeout"

class HTTPVersionNotSupported(ClientException):
    """
    HTTP 505 - The server does not support the HTTP protocol version used in the request.
    """
    http_status = 505
    message = "Version Not Supported"


# In Python 2.4 Exception is old-style and thus doesn't have a __subclasses__()
# so we can do this:
#     _code_map = dict((c.http_status, c)
#                      for c in ClientException.__subclasses__())
#
# Instead, we have to hardcode it:
_code_map = dict((c.http_status, c) for c in [HTTPBadRequest, HTTPUnauthorized,
                   HTTPForbidden, HTTPNotFound, HTTPMethodNotAllowed,
                   HTTPNotAcceptable, HTTPProxyAuthRequired, HTTPRequestTimeout, 
                   HTTPConflict, HTTPGone, HTTPLengthRequired,
                   HTTPPreconditionFailed, HTTPRequestEntityTooLarge,
                   HTTPRequestURITooLong, HTTPUnsupportedMediaType,
                   HTTPRequestedRangeNotSatisfiable, HTTPExpectationFailed,
                   HTTPTeaPot, HTTPServerError,
                   HTTPNotImplemented, HTTPBadGateway,
                   HTTPServiceUnavailable, HTTPGatewayTimeout,
                   HTTPVersionNotSupported])


def from_response(response, body):
    """
    Return an instance of an ClientException or subclass
    based on an httplib2 response.

    Usage:: 

        resp, body = http.request(...)
        if resp.status != 200:
            raise exception_from_response(resp, body)

    """
    cls = _code_map.get(response.status, ClientException)
    return cls(body)



# vim: tabstop=4 shiftwidth=4 softtabstop=4
#
# Copyright 2013 Hewlett Packard Development Company, L.P.
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.
"""
HP LeftHand REST Client

:Author: Kurt Martin
:Author: Walter A. Boring IV
:Copyright: Copyright 2013, Hewlett Packard Development Company, L.P.
:License: Apache v2.0

"""

version_tuple = (1, 0, 0)


def get_version_string():
    if isinstance(version_tuple[-1], basestring):
        return '.'.join(map(str, version_tuple[:-1])) + version_tuple[-1]
    return '.'.join(map(str, version_tuple))

version = get_version_string()
"""Current version of HPLeftHandClient."""



# vim: tabstop=4 shiftwidth=4 softtabstop=4
# Copyright 2009-2012 10gen, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""Test class of LeftHand Client handling volumes & snapshots """

import sys
import os
sys.path.insert(0, os.path.realpath(os.path.abspath('../')))

from hplefthandclient import exceptions
import test_HPLeftHandClient_base

VOLUME_NAME1 = 'VOLUME1_UNIT_TEST'
VOLUME_NAME2 = 'VOLUME2_UNIT_TEST'
VOLUME_NAME3 = 'VOLUME3_UNIT_TEST'
SNAP_NAME1 = 'SNAP_UNIT_TEST'


class HPLeftHandClientVolumeTestCase(test_HPLeftHandClient_base.
                                     HPLeftHandClientBaseTestCase):

    cluster_id = 0
    GB_TO_BYTES = 1073741824

    def setUp(self):
        super(HPLeftHandClientVolumeTestCase, self).setUp()

        try:
            cluster_info = self.cl.getClusterByName(
                test_HPLeftHandClient_base.
                HPLeftHandClientBaseTestCase.cluster)
            self.cluster_id = cluster_info['id']
        except Exception:
            pass

    def tearDown(self):

        try:
            volume_info = self.cl.getVolumeByName(VOLUME_NAME1)
            self.cl.deleteVolume(volume_info['id'])
        except Exception as ex:
            print ex
            pass
        try:
            volume_info = self.cl.getVolumeByName(VOLUME_NAME2)
            self.cl.deleteVolume(volume_info['id'])
        except Exception as ex:
            print ex
            pass

        super(HPLeftHandClientVolumeTestCase, self).tearDown()

    def test_1_create_volume(self):
        self.printHeader('create_volume')

        try:
            #add one
            optional = {'description': 'test volume',
                        'isThinProvisioned': True}
            self.cl.createVolume(VOLUME_NAME1, self.cluster_id,
                                 1 * self.GB_TO_BYTES, optional)
        except Exception as ex:
            print ex
            self.fail('Failed to create volume')
            return

        try:
            #check
            vol1 = self.cl.getVolumeByName(VOLUME_NAME1)
            self.assertIsNotNone(vol1)
            volName = vol1['name']
            self.assertEqual(VOLUME_NAME1, volName)

        except Exception as ex:
            print ex
            self.fail('Failed to get volume')
            return

        try:
            #add another
            optional = {'description': 'test volume2',
                        'isThinProvisioned': True}
            self.cl.createVolume(VOLUME_NAME2, self.cluster_id,
                                 1 * self.GB_TO_BYTES, optional)
        except Exception as ex:
            print ex
            self.fail('Failed to create volume')
            return

        try:
            #check
            vol2 = self.cl.getVolumeByName(VOLUME_NAME2)
            self.assertIsNotNone(vol2)
            volName = vol2['name']
            self.assertEqual(VOLUME_NAME2, volName)
        except Exception as ex:
            print ex
            self.fail("Failed to get volume")

        self.printFooter('create_volume')

    def test_1_create_volume_duplicate_name(self):
        self.printHeader('create_volume_duplicate_name')

        #add one and check
        try:
            optional = {'description': 'test volume',
                        'isThinProvisioned': True}
            self.cl.createVolume(VOLUME_NAME1, self.cluster_id,
                                 2 * self.GB_TO_BYTES, optional)
        except Exception as ex:
            print ex
            self.fail("Failed to create volume")

        self.assertRaises(exceptions.HTTPServerError,
                          self.cl.createVolume,
                          VOLUME_NAME1,
                          self.cluster_id,
                          2 * self.GB_TO_BYTES,
                          optional)
        self.printFooter('create_volume_duplicate_name')

    def test_1_create_volume_tooLarge(self):
        self.printHeader('create_volume_tooLarge')
        optional = {'description': 'test volume',
                    'isThinProvisioned': False}

        self.assertRaises(exceptions.HTTPServerError,
                          self.cl.createVolume,
                          VOLUME_NAME1,
                          self.cluster_id,
                          16777218 * self.GB_TO_BYTES,
                          optional)

        self.printFooter('create_volume_tooLarge')

    def test_2_get_volume_bad(self):
        self.printHeader('get_volume_bad')

        self.assertRaises(exceptions.HTTPNotFound,
                          self.cl.getVolumeByName,
                          'NoSuchVolume')

        self.printFooter('get_volume_bad')

    def test_2_get_volumes(self):
        self.printHeader('get_volumes')

        self.cl.createVolume(VOLUME_NAME1, self.cluster_id,
                             3 * self.GB_TO_BYTES)
        self.cl.createVolume(VOLUME_NAME2, self.cluster_id,
                             3 * self.GB_TO_BYTES)

        vol1 = self.cl.getVolumeByName(VOLUME_NAME1)
        vol2 = self.cl.getVolumeByName(VOLUME_NAME2)

        vols = self.cl.getVolumes()

        self.assertTrue(self.findInDict(vols['members'], 'name', vol1['name']))
        self.assertTrue(self.findInDict(vols['members'], 'name', vol2['name']))

        self.printFooter('get_volumes')

    def test_3_delete_volume_nonExist(self):
        self.printHeader('delete_volume_nonExist')
        volume_id = -1

        self.assertRaises(exceptions.HTTPServerError,
                          self.cl.deleteVolume,
                          volume_id)
        self.printFooter('delete_volume_nonExist')

    def test_3_delete_volumes(self):
        self.printHeader('delete_volumes')

        try:
            optional = {'description': 'Made by flask.'}
            self.cl.createVolume(VOLUME_NAME1, self.cluster_id,
                                 1 * self.GB_TO_BYTES, optional)
            vol1 = self.cl.getVolumeByName(VOLUME_NAME1)
            self.printHeader('vol1 id %s' % vol1['id'])
            self.printHeader('members %s' % vol1)
        except Exception as ex:
            print ex
            self.fail('Failed to create volume %s' % VOLUME_NAME1)

        try:
            optional = {'description': 'Made by flask.'}
            self.cl.createVolume(VOLUME_NAME2, self.cluster_id,
                                 1 * self.GB_TO_BYTES, optional)
            vol2 = self.cl.getVolumeByName(VOLUME_NAME2)
            self.printHeader('vol2 id %s' % vol2['id'])
        except Exception as ex:
            print ex
            self.fail('Failed to create volume %s' % VOLUME_NAME2)

        try:
            self.cl.deleteVolume(vol1['id'])
        except Exception as ex:
            print ex
            self.fail('Failed to delete %s' % vol1['id'])

        self.assertRaises(exceptions.HTTPNotFound,
                          self.cl.getVolumeByName,
                          VOLUME_NAME1)

        try:
            self.cl.deleteVolume(vol2['id'])
        except Exception as ex:
            print ex
            self.fail('Failed to delete %s' % vol2['id'])

        self.assertRaises(exceptions.HTTPNotFound,
                          self.cl.getVolumeByName,
                          VOLUME_NAME2)
        self.printFooter('delete_volumes')

    def test_4_create_snapshot(self):
        self.printHeader('create_snapshot')

        try:
            self.cl.createVolume(VOLUME_NAME1, self.cluster_id,
                                 1 * self.GB_TO_BYTES)
            volume_info = self.cl.getVolumeByName(VOLUME_NAME1)
            option = {'inheritAccess': True}
            self.cl.createSnapshot(SNAP_NAME1, volume_info['id'], option)
        except Exception as ex:
            print ex
            self.fail("Failed with unexpected exception")

        snap_info = self.cl.getSnapshotByName(SNAP_NAME1)
        self.cl.deleteSnapshot(snap_info['id'])
        self.printFooter('create_snapshot')

    def test_4_create_snapshot_nonExistVolume(self):
        self.printHeader('create_snapshot_nonExistVolume')

        optional = {'description': 'test snapshot'}
        self.assertRaises(exceptions.HTTPServerError,
                          self.cl.createSnapshot,
                          'UnitTestSnapshot',
                          -1,
                          optional)
        self.printFooter('create_snapshot_nonExistVolume')
#testing
#suite = unittest.TestLoader().loadTestsFromTestCase(HPLeftHandClientVolumeTestCase)
#unittest.TextTestRunner(verbosity=2).run(suite)



import argparse
from os import sys
import os
import pprint

# this is a hack to get the hp driver module
# and it's utils module on the search path.
cmd_folder = os.path.realpath(os.path.abspath(".."))
if cmd_folder not in sys.path:
    sys.path.insert(0, cmd_folder)

from hplefthandclient import client, exceptions

parser = argparse.ArgumentParser()
parser.add_argument("-debug", help="Turn on http debugging", default=False,
                    action="store_true")
args = parser.parse_args()


cl = client.HPLeftHandClient("http://10.10.22.7:8080/lhos")
# This is the local flask server url
#cl = client.HPLeftHandClient("http://127.0.0.1:5000/lhos")
if "debug" in args and args.debug == True:
    cl.debug_rest(True)


def test_login():
    print "Test Login"
    try:
        cl.login("administrator", "hpinvent")
        pprint.pprint("Login worked")
    except exceptions.HTTPUnauthorized:
        pprint.pprint("Login Failed")


def test_logout():
    print "Test Logout"
    try:
        cl.login("administrator", "hpinvent")
        pprint.pprint("Login worked")
    except exceptions.HTTPUnauthorized:
        pprint.pprint("Login Failed")

    try:
        cl.logout()
        pprint.pprint("Logout worked")
    except exceptions.HTTPUnauthorized:
        pprint.pprint("Logout Failed")


def test_get_snapshot(snap_id):
    print "Get Snapshot"
    try:
        cl.login("administrator", "hpinvent")
        snap = cl.getSnapshot(snap_id)
        pprint.pprint(snap)
    except exceptions.HTTPUnauthorized as ex:
        pprint.pprint("You must login first")
    except Exception as ex:
        pprint.pprint(ex)


def test_get_snapshot_by_name(name):
    print "Get Snapshot By Name"
    try:
        cl.login("administrator", "hpinvent")
        snap = cl.getSnapshotByName(name)
        pprint.pprint(snap)
    except exceptions.HTTPUnauthorized as ex:
        pprint.pprint("You must login first")
    except Exception as ex:
        pprint.pprint(ex)


def test_create_snapshot():
    print "Create Snapshot"
    try:
        cl.login("administrator", "hpinvent")
        vol = cl.createVolume("VolumeSource", 20, 1048576)
        snapshot = cl.createSnapshot('VolumeSnapshot', vol['id'])
        pprint.pprint(snapshot)
        cl.deleteSnapshot(snapshot['id'])
        cl.deleteVolume(vol['id'])
    except exceptions.HTTPUnauthorized:
        pprint.pprint("You must login first")


def test_get_snapshots():
    print "Get Snapshots"
    try:
        cl.login("administrator", "hpinvent")
        snapshots = cl.getSnapshots()
        pprint.pprint(snapshots)
    except exceptions.HTTPUnauthorized:
        pprint.pprint("You must login first")


def test_get_servers():
    print "Get Servers"
    try:
        cl.login("administrator", "hpinvent")
        servers = cl.getServers()
        pprint.pprint(servers)
    except exceptions.HTTPUnauthorized:
        pprint.pprint("You must login first")


def test_get_server(server_id):
    print "Get Server"
    try:
        cl.login("administrator", "hpinvent")
        server = cl.getServer(server_id)
        pprint.pprint(server)
    except exceptions.HTTPUnauthorized:
        pprint.pprint("You must login first")


def test_get_server_by_name(name):
    print "Get Server By Name"
    try:
        cl.login("administrator", "hpinvent")
        server = cl.getServerByName(name)
        pprint.pprint(server)
    except exceptions.HTTPUnauthorized:
        pprint.pprint("You must login first")


def test_get_volume(volume_id):
    print "Get Volumes"
    try:
        cl.login("administrator", "hpinvent")
        volume = cl.getVolume(volume_id)
        return volume
    except exceptions.HTTPUnauthorized:
        pprint.pprint("You must login first")


def test_get_volume_by_name(name):
    print "Get Volume By Name"
    try:
        cl.login("administrator", "hpinvent")
        volume = cl.getVolumeByName(name)
        return volume
    except exceptions.HTTPUnauthorized:
        pprint.pprint("You must login first")


def test_get_volumes():
    print "Get Volumes"
    try:
        cl.login("administrator", "hpinvent")
        volumes = cl.getVolumes()
        pprint.pprint(volumes)
    except exceptions.HTTPUnauthorized:
        pprint.pprint("You must login first")


def test_get_clusters():
    print "Get Clusters"
    try:
        cl.login("administrator", "hpinvent")
        volumes = cl.getClusters()
        pprint.pprint(volumes)
    except exceptions.HTTPUnauthorized:
        pprint.pprint("You must login first")


def test_get_cluster_by_name(name):
    print "Get Cluster By Name"
    try:
        cl.login("administrator", "hpinvent")
        volumes = cl.getClusterByName(name)
        pprint.pprint(volumes)
    except exceptions.HTTPUnauthorized:
        pprint.pprint("You must login first")


def test_create_volume():
    print "Create Volumes"
    try:
        cl.login("administrator", "hpinvent")
        vol = cl.createVolume("Volume1", 20, 1048576)
        return vol

    except exceptions.HTTPUnauthorized:
        pprint.pprint("You must login first")


def test_delete_volume(volume_id):
    print "Delete a Volume"
    try:
        cl.login("administrator", "hpinvent")
        vol = cl.deleteVolume(volume_id)
        return vol
    except exceptions.HTTPUnauthorized:
        pprint.pprint("You must login first")


def test_error():
    print "test Error"
    try:
        cl.login("administrator", "hpinvent")
        resp, body = cl.http.get('/throwerror')
        pprint.pprint(resp)
        pprint.pprint(body)
    except Exception as ex:
        print ex

#test_login()
#test_logout()

#vols = test_get_volumes()
#vol = test_get_volume_by_name("vol1_test")
#vols = test_get_clusters()

#snap = test_get_snapshot("25")
#snap = test_get_snapshot_by_name("vol1_test_SS_1")
snaps = test_get_snapshots()

#clusters = test_get_clusters()
#cluster = test_get_cluster_by_name("ClusterVSA309")

#servers = test_get_servers()
#server = test_get_server_by_name("jim-devstack")

#vol = test_get_volume(23)
#pprint.pprint(vol)

#test_delete_volume(400)

#test_create_snapshot()

#test_error()



# vim: tabstop=4 shiftwidth=4 softtabstop=4
#
# Copyright 2012 Hewlett Packard Development Company, L.P.
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.
"""
HPLeftHand REST Client

.. module: HPLeftHandClient
.. moduleauthor: Kurt Martin

:Author: Kurt Martin
:Description: This is the LeftHand/StoreVirtual Client that talks to the
LeftHand OS REST Service.

This client requires and works with version 11.5 of the LeftHand firmware

"""

from hplefthandclient import http


class HPLeftHandClient:

    def __init__(self, api_url):
        self.api_url = api_url
        self.http = http.HTTPJSONRESTClient(self.api_url)

    def debug_rest(self, flag):
        """
        This is useful for debugging requests to LeftHand 

        :param flag: set to True to enable debugging
        :type flag: bool

        """
        self.http.set_debug_flag(flag)

    def login(self, username, password):
        """
        This authenticates against the LH OS REST server and creates a session.

        :param username: The username
        :type username: str
        :param password: The password
        :type password: str

        :returns: None

        """
        self.http.authenticate(username, password)

    def logout(self):
        """
        This destroys the session and logs out from the LH OS server

        :returns: None

        """
        self.http.unauthenticate()

    def getClusters(self):
        """
        Get the list of Clusters

        :returns: list of Clusters
        """
        response, body = self.http.get('/clusters')
        return body

    def getCluster(self, cluster_id):
        """
        Get information about a Cluster

        :param cluster_id: The id of the cluster to find
        :type cluster_id: str

        :returns: cluster
        """
        response, body = self.http.get('/clusters/%s' % cluster_id)
        return body

    def getClusterByName(self, name):
        """
        Get information about a cluster by name

        :param name: The name of the cluster to find
        :type name: str

        :returns: cluster
        :raises: :class:`~hplefthandclient.exceptions.HTTPNotFound` -
        NON_EXISTENT_CLUSTER - cluster doesn't exist
        """
        response, body = self.http.get('/clusters?name=%s' % name)
        return body

    def getServers(self):
        """
        Get the list of Servers

        :returns: list of Servers
        """
        response, body = self.http.get('/servers')
        return body

    def getServer(self, server_id):
        """
        Get information about a server

        :param server_id: The id of the server to find
        :type server_id: str

        :returns: server
        :raises: :class:`~hplefthandclient.exceptions.HTTPServerError`
        """
        response, body = self.http.get('/servers/%s' % server_id)
        return body

    def getServerByName(self, name):
        """
        Get information about a server by name

        :param name: The name of the server to find
        :type name: str

        :returns: server
        :raises: :class:`~hplefthandclient.exceptions.HTTPNotFound` -
        NON_EXISTENT_SERVER - server doesn't exist
        """
        response, body = self.http.get('/servers?name=%s' % name)
        return body

    def createServer(self, name, iqn, optional=None):
        """
        Create a server by name

        :param name: The name of the server to create
        :type name: str
        :param iqn: The iSCSI qualified name
        :type name: str
        :param optional: Dictionary of optional params
        :type optional: dict

        .. code-block:: python

            optional = {
                'description' : "some comment",
                'iscsiEnabled' : True,
                'chapName': "some chap name",
                'chapAuthenticationRequired': False,
                'chapInitiatorSecret': "initiator secret",
                'chapTargetSecret': "target secret",
                'iscsiLoadBalancingEnabled': True,
                'controllingServerName': "server name",
                'fibreChannelEnabled': False,
                'inServerCluster": True
            }

        :returns: server
        :raises: :class:`~hplefthandclient.exceptions.HTTPNotFound` -
        NON_EXISTENT_SERVER - server doesn't exist
        """
        info = {'name': name, 'iscsiIQN': iqn}
        if optional:
            info = self._mergeDict(info, optional)

        response, body = self.http.post('/servers', body=info)
        return body

    def deleteServer(self, server_id):
        """
        Delete a Server

        :param server_id: the server ID to delete

        :raises: :class:`~hplefthandclient.exceptions.HTTPNotFound` -
        NON_EXISTENT_SERVER - The server does not exist
        """
        response, body = self.http.delete('/servers/%s' % server_id)
        return body

    def getSnapshots(self):
        """
        Get the list of Snapshots

        :returns: list of Snapshots
        """
        response, body = self.http.get('/snapshots')
        return body

    def getSnapshot(self, snapshot_id):
        """
        Get information about a Snapshot

        :returns: snapshot
        :raises: :class:`~hplefthandclient.exceptions.HTTPServerError`
        """
        response, body = self.http.get('/snapshots/%s' % snapshot_id)
        return body

    def getSnapshotByName(self, name):
        """
        Get information about a snapshot by name

        :param name: The name of the snapshot to find

        :returns: volume
        :raises: :class:`~hplefthandclient.exceptions.HTTPNotFound` -
        NON_EXISTENT_SNAP - shapshot doesn't exist
        """
        response, body = self.http.get('/snapshots?name=%s' % name)
        return body

    def createSnapshot(self, name, source_volume_id, optional=None):
        """
        Create a snapshot of an existing Volume

        :param name: Name of the Snapshot
        :type name: str
        :param source_volume_id: The volume you want to snapshot
        :type source_volume_id: int
        :param optional: Dictionary of optional params
        :type optional: dict

        .. code-block:: python

            optional = {
                'description' : "some comment",
                'inheritAccess' : false
            }

        """
        parameters = {'name': name}
        if optional:
            parameters = self._mergeDict(parameters, optional)

        info = {'action': 'createSnapshot',
                'parameters': parameters}

        response, body = self.http.post('/volumes/%s' % source_volume_id,
                                        body=info)
        return body

    def deleteSnapshot(self, snapshot_id):
        """
        Delete a Snapshot

        :param snapshot_id: the snapshot ID to delete

        :raises: :class:`~hplefthandclient.exceptions.HTTPNotFound` -
        NON_EXISTENT_SNAPSHOT - The snapshot does not exist
        """
        response, body = self.http.delete('/snapshots/%s' % snapshot_id)
        return body

    def cloneSnapshot(self, name, source_snapshot_id, optional=None):
        """
        Create a clone of an existing Shapshot

        :param name: Name of the Snapshot clone
        :type name: str
        :param source_snapshot_id: The snapshot you want to clone
        :type source_snapshot_id: int
        :param optional: Dictionary of optional params
        :type optional: dict

        .. code-block:: python

            optional = {
                'description' : "some comment"
            }

        """
        parameters = {'name': name}
        if optional:
            parameters = self._mergeDict(parameters, optional)

        info = {'action': 'createSmartClone',
                'parameters': parameters}

        response, body = self.http.post('/snapshots/%s' % source_snapshot_id,
                                        body=info)
        return body

    def getVolumes(self):
        """
        Get the list of Volumes

        :returns: list of Volumes
        """
        response, body = self.http.get('/volumes')
        return body

    def getVolume(self, volume_id):
        """
        Get information about a volume

        :param volume_id: The id of the volume to find
        :type volume_id: str

        :returns: volume
        :raises: :class:`~hplefthandclient.exceptions.HTTPNotFound` -
        NON_EXISTENT_VOL - volume doesn't exist
        """
        response, body = self.http.get('/volumes/%s' % volume_id)
        return body

    def getVolumeByName(self, name):
        """
        Get information about a volume by name

        :param name: The name of the volume to find
        :type volume_id: str

        :returns: volume
        :raises: :class:`~hplefthandclient.exceptions.HTTPNotFound` -
        NON_EXISTENT_VOL - volume doesn't exist
        """
        response, body = self.http.get('/volumes?name=%s' % name)
        return body

    def createVolume(self, name, cluster_id, size, optional=None):
        """ Create a new volume

        :param name: the name of the volume
        :type name: str
        :param cluster_id: the cluster Id
        :type cluster_id: int
        :param sizeKB: size in KB for the volume
        :type sizeKB: int
        :param optional: dict of other optional items
        :type optional: dict

        .. code-block:: python

            optional = {
             'description': 'some comment',
             'isThinProvisioned': 'true',
             'autogrowSeconds': 200,
             'clusterName': 'somename',
             'isAdaptiveOptimizationEnabled': 'true',
             'dataProtectionLevel': 2,
            }

        :returns: List of Volumes

        :raises: :class:`~hplefthandclient.exceptions.HTTPConflict` -
        EXISTENT_SV - Volume Exists already
        """
        info = {'name': name, 'clusterID': cluster_id, 'size': size}
        if optional:
            info = self._mergeDict(info, optional)

        response, body = self.http.post('/volumes', body=info)
        return body

    def deleteVolume(self, volume_id):
        """
        Delete a volume

        :param name: the name of the volume
        :type name: str

        :raises: :class:`~hplefthandclient.exceptions.HTTPNotFound` -
        NON_EXISTENT_VOL - The volume does not exist
        """
        response, body = self.http.delete('/volumes/%s' % volume_id)
        return body

    def modifyVolume(self, volume_id, optional):
        """Modify an existing volume.

        :param volume_id: The id of the volume to find
        :type volume_id: str

        :returns: volume
        :raises: :class:`~hplefthandclient.exceptions.HTTPNotFound` -
        NON_EXISTENT_VOL - volume doesn't exist
        """
        info = {'volume_id': volume_id}
        info = self._mergeDict(info, optional)
        response, body = self.http.put('/volumes/%s' % volume_id, body=info)
        return body

    def cloneVolume(self, name, source_volume_id, optional=None):
        """
        Create a clone of an existing Volume

        :param name: Name of the Volume clone
        :type name: str
        :param source_volume_id: The Volume you want to clone
        :type source_volume_id: int
        :param optional: Dictionary of optional params
        :type optional: dict

        .. code-block:: python

            optional = {
                'description' : "some comment"
            }

        """
        parameters = {'name': name}
        if optional:
            parameters = self._mergeDict(parameters, optional)

        info = {'action': 'createSmartClone',
                'parameters': parameters}

        response, body = self.http.post('/volumes/%s' % source_volume_id,
                                        body=info)
        return body

    def addServerAccess(self, volume_id, server_id, optional=None):
        """
        Assign a Volume to a Server

        :param volume_id: Volume ID of the volume
        :type name: int
        :param server_id: Server ID of the server to add the volume to
        :type source_volume_id: int
        :param optional: Dictionary of optional params
        :type optional: dict

        .. code-block:: python

            optional = {
                'Transport' : 0,
                'Lun' : 1,
            }

        """
        parameters = {'serverID': server_id,
                      'exclusiveAccess': True,
                      'readAccess': True,
                      'writeAccess': True}
        if optional:
            parameters = self._mergeDict(parameters, optional)

        info = {'action': 'addServerAccess',
                'parameters': parameters}

        response, body = self.http.post('/volumes/%s' % volume_id,
                                        body=info)
        return body

    def removeServerAccess(self, volume_id, server_id):
        """
        Unassign a Volume from a Server

        :param volume_id: Volume ID of the volume
        :type name: int
        :param server_id: Server ID of the server to remove the volume fom
        :type source_volume_id: int

        """
        parameters = {'serverID': server_id}

        info = {'action': 'removeServerAccess',
                'parameters': parameters}

        response, body = self.http.post('/volumes/%s' % volume_id,
                                        body=info)
        return body

    def _mergeDict(self, dict1, dict2):
        """
        Safely merge 2 dictionaries together

        :param dict1: The first dictionary
        :type dict1: dict
        :param dict2: The second dictionary
        :type dict2: dict

        :returns: dict

        :raises Exception: dict1, dict2 is not a dictionary
        """
        if type(dict1) is not dict:
            raise Exception("dict1 is not a dictionary")
        if type(dict2) is not dict:
            raise Exception("dict2 is not a dictionary")

        dict3 = dict1.copy()
        dict3.update(dict2)
        return dict3



import hplefthandclient

try:
    from setuptools import setup, find_packages
except ImportError:
    from distutils.core import setup, find_packages
import sys


setup(
  name='hplefthandclient',
  version=hplefthandclient.version,
  description="HP LeftHand/StoreVirtual HTTP REST Client",
  author="Kurt Martin",
  author_email="kurt.f.martin@hp.com",
  maintainer="Kurt Martin",
  keywords=["hp", "lefthand", "storevirtual", "rest"],
  requires=['httplib2(>=0.6.0)'],
  install_requires=['httplib2 >= 0.6.0'],
  tests_require=["nose", "werkzeug", "nose-testconfig"],
  license="Apache License, Version 2.0",
  packages=find_packages(),
  provides=['hplefthandclient'],
  url="http://packages.python.org/hplefthandclient",
  classifiers=[
     'Development Status :: 3 - Alpha',
     'Intended Audience :: Developers',
     'License :: OSI Approved :: Apache Software License',
     'Environment :: Web Environment',
     'Programming Language :: Python',
     'Programming Language :: Python :: 2.6',
     'Programming Language :: Python :: 2.7',
     'Topic :: Internet :: WWW/HTTP',
     
     ]
  )



import argparse
from os import sys
import random
from sys import path
from os import getcwd
import os, sys, inspect, pprint

# this is a hack to get the hp driver module
# and it's utils module on the search path.
cmd_folder = os.path.realpath(os.path.abspath("..") )
if cmd_folder not in sys.path:
     sys.path.insert(0, cmd_folder)

from hplefthandclient import client, exceptions


def get_volumes(cl):
    print "Get Volumes"
    try:
       volumes = cl.getVolumes()
       if volumes:
           for volume in volumes['members']:
               print "Found '%s'" % volume['name']
    except exceptions.HTTPUnauthorized as ex:
       print "You must login first"
    except Exception as ex:
       print ex
    print "Complete\n"

def get_volume(cl, name):
    print "Get Volume %s" % name
    try:
        vol = cl.getVolume(name)
        pprint.pprint(vol)
    except exceptions.HTTPUnauthorized as ex:
       print "You must login first"
    except Exception as ex:
       print ex
    print "Complete\n"

def get_hosts(cl):
    print "Get Hosts"
    try:
       hosts = cl.getHosts()
       if hosts:
           for host in hosts['members']:
               pprint.pprint(host)
#               print "Found '%s'" % host['name']
    except exceptions.HTTPUnauthorized as ex:
       print "You must login first"
    except Exception as ex:
       print ex

def get_host(cl,hostname):
    try:
        host = cl.getHost(hostname)
        pprint.pprint(host)
    except exceptions.HTTPUnauthorized as ex:
       print "You must login first"
    except Exception as ex:
       print ex

def delete_host(cl,hostname):
    try:
        host = cl.deleteHost(hostname)
    except exceptions.HTTPUnauthorized as ex:
       print "You must login first"
    except Exception as ex:
       print ex

def get_host_vluns(cl,hostname):
    try:
        host = cl.getHostVLUNs(hostname)
        pprint.pprint(host)
    except exceptions.HTTPUnauthorized as ex:
       print "You must login first"
    except Exception as ex:
       print ex

def delete_host_vluns(cl, hostName):
    try:
        vluns = cl.getHostVLUNs(hostName)
        if vluns:
            for vlun in vluns:
                print "Deleting VLUN %s " % vlun['volumeName']
                cl.deleteVLUN(vlun['volumeName'], vlun['lun'], 
                              vlun['hostname'], vlun['portPos'])

    except exceptions.HTTPUnauthorized as ex:
        print "You must login"
    except Exception as ex:
        print ex

def get_ports(cl):
    try:
        ports = cl.getPorts()
        pprint.pprint(ports)
    except exceptions.HTTPUnauthorized as ex:
       print "You must login first"
    except Exception as ex:
       print ex


def get_vluns(cl):
    print "Get VLUNs"
    try:
        vluns = cl.getVLUNs()
        if vluns:
            pprint.pprint(vluns)
            for vlun in vluns['members']:
                pprint.pprint(vlun)
                print "Found VLUN '%s'" % vlun['volumeName']
    except exceptions.HTTPUnauthorized as ex:
       print "You must login first"
    except Exception as ex:
       print ex


def get_vlun(cl,vlunname):
    try:
        vlun = cl.getVLUN(vlunname)
        pprint.pprint(vlun)
    except exceptions.HTTPUnauthorized as ex:
       print "You must login first"
    except Exception as ex:
       print ex


def get_cpgs(cl):
    print "Get CPGs"
    try:
        cpgs = cl.getCPGs()
        if cpgs:
            for cpg in cpgs['members']:
                print "Found CPG '%s'" % cpg['name']
    except exceptions.HTTPUnauthorized as ex:
       print "You must login first"
    except Exception as ex:
       print ex

def get_cpg(cl, name):
    try:
        cpg = cl.getCPG(name)
        pprint.pprint(cpg)
    except exceptions.HTTPUnauthorized as ex:
       print "You must login first"
    except Exception as ex:
       print ex



import pprint
import json
import random
import string
import argparse
from werkzeug.exceptions import default_exceptions
from werkzeug.exceptions import HTTPException

parser = argparse.ArgumentParser()
parser.add_argument("-debug", help="Turn on http debugging",
                    default=False, action="store_true")
parser.add_argument("-user", help="User name", default='administrator')
parser.add_argument("-password", help="User password", default='hpinvent')
parser.add_argument("-port", help="Port to listen on", type=int, default=5001)
args = parser.parse_args()
user_name = args.user
user_pass = args.password
debugRequests = False
if "debug" in args and args.debug is True:
    debugRequests = True

#__all__ = ['make_json_app']


def id_generator(size=6, chars=string.ascii_uppercase + string.digits):
    return ''.join(random.choice(chars) for x in range(size))


def make_json_app(import_name, **kwargs):
    """
    Creates a JSON-oriented Flask app.

    All error responses that you don't specifically
    manage yourself will have application/json content
    type, and will contain JSON like this (just an example):

    { "message": "405: Method Not Allowed" }
    """
    def make_json_error(ex):
        pprint.pprint(ex)
        pprint.pprint(ex.code)
        #response = jsonify(message=str(ex))
        response = jsonify(ex)
        response.status_code = (ex.code
                                if isinstance(ex, HTTPException)
                                else 500)
        return response

    app = Flask(import_name, **kwargs)
    #app.debug = True
    app.secret_key = id_generator(24)

    for code in default_exceptions.iterkeys():
        app.error_handler_spec[None][code] = make_json_error

    return app

app = make_json_app(__name__)

session_key = id_generator(24)


def debugRequest(request):
    if debugRequests:
        print "\n"
        pprint.pprint(request)
        pprint.pprint(request.headers)
        pprint.pprint(request.data)


def throw_error(http_code, error_code=None,
                desc=None, debug1=None, debug2=None):
    if error_code:
        info = {'code': error_code, 'desc': desc}
        if debug1:
            info['debug1'] = debug1
        if debug2:
            info['debug2'] = debug2
        abort(Response(json.dumps(info), status=http_code))
    else:
        abort(http_code)


@app.route('/')
def index():
    debugRequest(request)
    if 'username' in session:
        return 'Logged in as %s' % escape(session['username'])
    abort(401)


@app.route('/lhos/throwerror')
def errtest():
    debugRequest(request)
    throw_error(405, 'ERR_TEST', 'testing throwing an error',
                'debug1 message', 'debug2 message')


@app.errorhandler(404)
def not_found(error):
    debugRequest(request)
    return Response("%s has not been implemented" % request.path, status=501)


@app.route('/lhos/credentials', methods=['GET', 'POST'])
def credentials():
    debugRequest(request)

    if request.method == 'GET':
        return 'GET credentials called'

    elif request.method == 'POST':
        data = json.loads(request.data)

        if data['user'] == user_name and data['password'] == user_pass:
            #do something good here
            try:
                resp = make_response(json.dumps({'key': session_key}), 201)
                resp.headers['Location'] = '/lhos/credentials/%s' % session_key
                session['username'] = data['user']
                session['password'] = data['password']
                session['session_key'] = session_key
                return resp
            except Exception as ex:
                pprint.pprint(ex)

        else:
            #authentication failed!
            throw_error(401, "HTTP_AUTH_FAIL",
                        "Username and or Password was incorrect")


@app.route('/lhos/credentials/<session_key>', methods=['DELETE'])
def logout_credentials(session_key):
    debugRequest(request)
    session.clear()
    return 'DELETE credentials called'

#### CLUSTER INFO ####


@app.route('/lhos/clusters', methods=['GET'])
def get_cluster_by_name():
    debugRequest(request)
    cluster_name = request.args.get('name')

    for cluster in clusters['members']:
        if cluster['name'] == cluster_name:
            resp = make_response(json.dumps(cluster), 200)
            return resp

    throw_error(404, 'NON_EXISTENT_CLUSTER', "cluster doesn't exist")

### VOLUMES & SNAPSHOTS ####


@app.route('/lhos/volumes', methods=['GET'])
def get_volume():
    debugRequest(request)
    volume_name = None
    volume_name = request.args.get('name')

    if volume_name is not None:
        for volume in volumes['members']:
            if volume['name'] == volume_name:
                resp = make_response(json.dumps(volume), 200)
                return resp
        throw_error(404, 'NON_EXISTENT_VOLUME', "volume doesn't exist")
    else:
        resp = make_response(json.dumps(volumes), 200)
        return resp


@app.route('/lhos/snapshots', methods=['GET'])
def get_snapshot():
    debugRequest(request)
    snapshot_name = None
    snapshot_name = request.args.get('name')

    if snapshot_name is not None:
        pprint.pprint('snapshot name %s' % snapshot_name)
        for snapshot in snapshots['members']:
            if snapshot['name'] == snapshot_name:
                pprint.pprint('snapshot namei inside %s' % snapshot['name'])
                resp = make_response(json.dumps(snapshot), 200)
                return resp
        throw_error(404, 'NON_EXISTENT_SNAPSHOT', "snapshot doesn't exist")
    else:
        resp = make_response(json.dumps(snapshots), 200)
        return resp


@app.route('/lhos/volumes/<volume_id>', methods=['POST'])
def create_snapshot(volume_id):
    debugRequest(request)
    data = json.loads(request.data)

    valid_keys = {'action': None, 'parameters': None}

    ## do some fake errors here depending on data
    for key in data.keys():
        if key not in valid_keys.keys():
            throw_error(400, 'INV_INPUT', "Invalid Parameter '%s'" % key)

    for volume in volumes['members']:
        if volume['id'] == int(volume_id):
            snapshots['members'].append({'name': data['parameters'].get('name'),
                                         'id': random.randint(1, 2000)})
            pprint.pprint(snapshots)
            return make_response("", 200)

    throw_error(500, 'SERVER_ERROR', "volume doesn't exist")


@app.route('/lhos/volumes', methods=['POST'])
def create_volumes():
    debugRequest(request)
    data = json.loads(request.data)

    valid_keys = {'name': None, 'isThinProvisioned': None, 'size': None,
                  'description': None, 'clusterID': None}

    for key in data.keys():
        if key not in valid_keys.keys():
            throw_error(500, 'SERVER_ERROR', "Invalid Parameter '%s'" % key)

    if 'name' in data.keys():
        for vol in volumes['members']:
            if vol['name'] == data['name']:
                throw_error(500, 'SERVER_ERROR',
                            'The volume already exists.')
    else:
        throw_error(500, 'SERVER_ERROR',
                    'No volume name provided.')

    if 'size' in data.keys():
        if data['size'] > 17592188141567:
            throw_error(500, 'SERVER_ERROR', 'Volume to larger')

    data['id'] = random.randint(1, 2000)

    volumes['members'].append(data)
    return make_response("", 200)


@app.route('/lhos/volumes/<volume_id>', methods=['DELETE'])
def delete_volumes(volume_id):
    debugRequest(request)
    for volume in volumes['members']:
        if volume['id'] == int(volume_id):
            volumes['members'].remove(volume)
            return make_response("", 200)

    throw_error(500, 'SERVER_ERROR',
                "The volume id '%s' does not exists." % volume_id)


@app.route('/lhos/snapshots/<snapshot_id>', methods=['DELETE'])
def delete_snapshots(snapshot_id):
    debugRequest(request)
    for snapshot in snapshots['members']:
        if snapshot['id'] == int(snapshot_id):
            snapshots['members'].remove(snapshot)
            return make_response("", 200)

    throw_error(404, 'NON_EXISTENT_SNAPSHOT',
                "The snapshot id '%s' does not exists." % snapshot_id)


if __name__ == "__main__":

    #fake  volumes
    global volumes
    volumes = {'members': [{
               'autogrowSeconds': 2,
               'bytesWritten': 0,
               'clusterId': 21,
               'clusterName': 'ClusterVSA309',
               'created': '2013-10-23T16:58:58Z',
               'dataProtectionLevel': 0,
               'dataWritten': 0,
               'description': 'test volume',
               'fcTransportStatus': 0,
               'fibreChannelPaths': None,
               'friendlyName': '',
               'hasUnrecoverableIOErrors': False,
               'id': 24,
               'isAdaptiveOptimizationEnabled': True,
               'isAvailable': True,
               'isDeleting': False,
               'isLicensed': True,
               'isMigrating': False,
               'isPrimary': True,
               'isThinProvisioned': False,
               'isVIPRebalancing': False,
               'iscsiIqn': 'iqn.2003-10.com.lefthandnetworks:mgvsa309:24:vol1',
               'iscsiSessions': None,
               'migrationStatus': 'none',
               'modified': '',
               'name': 'VOLUME0_UNIT_TEST',
               'numberOfReplicas': 1,
               'provisionedSpace': 4194304,
               'replicationStatus': 'normal',
               'restripePendingStatus': 'none',
               'resynchronizationStatus': 'none',
               'scsiLUNStatus': 'available',
               'serialNumber': '27d18c785f81e91f36a5073fff9233720000000000000018',
               'size': 1048576,
               'snapshots': {'name': 'snapshots',
                             'resource': None,
                             'type': 'snapshot',
                             'uri': '/snapshots?volumeName=VOLUME1_UNIT_TEST'},
               'transport': 0,
               'transportServerId': 0,
               'type': 'volume',
               'uri': '/lhos/volumes/24',
               'volumeACL': None}],
               'total': 4}

    #fake snapshots
    global snapshots
    snapshots = {'members': [{
                 'autogrowSeconds': 2,
                 'bytesWritten': 0,
                 'clusterId': 21,
                 'clusterName': 'ClusterVSA309',
                 'created': '2013-10-23T16:59:19Z',
                 'dataWritten': 0,
                 'description': '',
                 'fcTransportStatus': 0,
                 'fibreChannelPaths': None,
                 'hasUnrecoverableIOErrors': False,
                 'id': 26,
                 'isAutomatic': False,
                 'isAvailable': True,
                 'isDeleting': False,
                 'isLicensed': True,
                 'isMigrating': False,
                 'isPrimary': True,
                 'isThinProvisioned': True,
                 'iscsiIqn': 'iqn.2003-10.com.lefthandnetworks:mgvsa309:26:vol1-ss-1',
                 'managedBy': 0,
                 'migrationStatus': 'none',
                 'modified': '',
                 'name': 'vol1_SS_1',
                 'provisionedSpace': 528384,
                 'replicationStatus': 'normal',
                 'restripePendingStatus': 'none',
                 'resynchronizationStatus': 'none',
                 'scsiLUNStatus': 'available',
                 'serialNumber': '27d18c785f81e91f36a5073fff923372000000000000001a',
                 'sessions': None,
                 'size': 4194304,
                 'snapshotACL': None,
                 'transport': 0,
                 'transportServerId': 0,
                 'type': 'snapshot',
                 'uri': '/lhos/snapshots/26',
                 'writableSpaceUsed': 0}],
                 'total': 4}

    #fake clusters
    global clusters
    clusters = {'members': [{'adaptiveOptimizationCapable': False,
                'created': 'N/A',
                'description': '',
                'id': 21,
                'modified': 'N/A',
                'moduleCount': 1,
                'name': 'ClusterVSA309',
                'spaceAvailable': 13457408,
                'spaceTotal': 40728576,
                'storageModuleIPAddresses': ['10.10.30.165'],
                'supportedFeatures': [''],
                'type': 'cluster',
                'uri': '/lhos/clusters/21',
                'virtualIPAddresses': [{'ipV4Address': '10.10.22.7',
                                        'ipV4NetMask': '255.255.224.0'}],
                'virtualIPEnabled': True,
                'volumeCreationSpace': [{'availableSpace': 13457408,
                                         'replicationLevel': 1}],
                'volumes': {'name': 'volumes',
                            'resource': None,
                            'type': 'volume',
                            'uri': '/volumes?clusterName=ClusterVSA309'}}],
                'name': 'Clusters Collection',
                'total': 1,
                'type': 'cluster',
                'uri': '/lhos/clusters'}

    app.run(port=args.port, debug=debugRequests)
